@@ -47,6 +47,7 @@
 #include <KWindowSystem>
 #include <KWindowEffects>
 #include <KLocalizedString>
+#include <KStatusNotifierItem>
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -55,7 +56,10 @@
 #include <QWhatsThis>
 #include <QWindow>
 #include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusPendingReply>
 #include <QPlatformSurfaceEvent>
+#include <QScreen>
 
 #if HAVE_X11
 #include <QX11Info>
@@ -87,8 +91,10 @@ MainWindow::MainWindow(QWidget* parent)
     m_sessionStack = new SessionStack(this);
     m_titleBar = new TitleBar(this);
     m_tabBar = new TabBar(this);
+    m_notifierItem = nullptr;
 
-    m_firstRunDialog = NULL;
+
+    m_firstRunDialog = nullptr;
     m_isFullscreen = false;
 
 #if HAVE_X11
@@ -99,8 +105,8 @@ MainWindow::MainWindow(QWidget* parent)
 #endif
     m_isWayland = KWindowSystem::isPlatformWayland();
 #if HAVE_KWAYLAND
-    m_plasmaShell = Q_NULLPTR;
-    m_plasmaShellSurface = Q_NULLPTR;
+    m_plasmaShell = nullptr;
+    m_plasmaShellSurface = nullptr;
     initWayland();
 #endif
 
@@ -114,6 +120,9 @@ MainWindow::MainWindow(QWidget* parent)
     connect(m_tabBar, SIGNAL(lastTabClosed()), this, SLOT(handleLastTabClosed()));
     connect(m_tabBar, SIGNAL(tabSelected(int)), m_sessionStack, SLOT(raiseSession(int)));
     connect(m_tabBar, SIGNAL(tabClosed(int)), m_sessionStack, SLOT(removeSession(int)));
+    connect(m_tabBar, &TabBar::tabTitleEdited, m_sessionStack, [&](int , QString) {
+                    m_sessionStack->raiseSession(m_sessionStack->activeSessionId());
+            });
     connect(m_tabBar, SIGNAL(requestTerminalHighlight(int)), m_sessionStack, SLOT(handleTerminalHighlightRequest(int)));
     connect(m_tabBar, SIGNAL(requestRemoveTerminalHighlight()), m_sessionStack, SIGNAL(removeTerminalHighlight()));
     connect(m_tabBar, SIGNAL(tabContextMenuClosed()), m_sessionStack, SIGNAL(removeTerminalHighlight()));
@@ -143,7 +152,6 @@ MainWindow::MainWindow(QWidget* parent)
     }
     else
     {
-        showStartupPopup();
         if (Settings::pollMouse()) toggleMousePoll(true);
     }
 
@@ -263,6 +271,10 @@ void MainWindow::setupActions()
     KGlobalAccel::self()->setGlobalShortcut(action, QList<QKeySequence>() << QKeySequence(Qt::Key_F11));
 #endif
     connect(action, SIGNAL(triggered()), this, SLOT(toggleWindowState()));
+    connect(action, SIGNAL(changed()), this, SLOT(updateTrayTooltip()));
+    connect(KGlobalAccel::self(), SIGNAL(globalShortcutChanged(QAction*, const QKeySequence&)),
+            this, SLOT(updateTrayTooltip()));
+    updateTrayTooltip();
 
     action = actionCollection()->addAction(QStringLiteral("keep-open"));
     action->setText(xi18nc("@action", "Keep window open when it loses focus"));
@@ -450,10 +462,18 @@ void MainWindow::setupActions()
 
     for (uint i = 1; i <= 10; ++i)
     {
-        action = actionCollection()->addAction(QString(QStringLiteral("switch-to-session-%1")).arg(i));
+        action = actionCollection()->addAction(QStringLiteral("switch-to-session-%1").arg(i));
         action->setText(xi18nc("@action", "Switch to Session %1", i));
-        action->setData(i);
+        action->setData(i - 1);
         connect(action, SIGNAL(triggered()), this, SLOT(handleSwitchToAction()));
+
+        if (i < 10) {
+            // add default shortcut bindings for the first 9 sessions
+            actionCollection()->setDefaultShortcut(action, QStringLiteral("Alt+%1").arg(i));
+        } else {
+            // add default shortcut bindings for the 10th session
+            actionCollection()->setDefaultShortcut(action, Qt::ALT + Qt::Key_0);
+        }
     }
 
     m_actionCollection->associateWidget(this);
@@ -616,18 +636,18 @@ void MainWindow::handleSwitchToAction()
     QAction* action = qobject_cast<QAction*>(QObject::sender());
 
     if (action && !action->data().isNull())
-        m_sessionStack->raiseSession(m_tabBar->sessionAtTab(action->data().toInt()-1));
+        m_sessionStack->raiseSession(m_tabBar->sessionAtTab(action->data().toInt()));
 }
 
 void MainWindow::setupMenu()
 {
-    m_menu->insertSection(0, xi18nc("@title:menu", "Help"));
+    m_menu->insertSection(nullptr, xi18nc("@title:menu", "Help"));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::WhatsThis))));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::ReportBug))));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::AboutApp))));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::AboutKDE))));
 
-    m_menu->insertSection(0, xi18nc("@title:menu", "Quick Options"));
+    m_menu->insertSection(nullptr, xi18nc("@title:menu", "Quick Options"));
     m_menu->addAction(actionCollection()->action(QStringLiteral("view-full-screen")));
     m_menu->addAction(actionCollection()->action(QStringLiteral("keep-open")));
 
@@ -646,7 +666,7 @@ void MainWindow::setupMenu()
     m_windowHeightMenu->setTitle(xi18nc("@title:menu", "Height"));
     m_menu->addMenu(m_windowHeightMenu);
 
-    m_menu->insertSection(0, xi18nc("@title:menu", "Settings"));
+    m_menu->insertSection(nullptr, xi18nc("@title:menu", "Settings"));
     m_menu->addAction(actionCollection()->action(QStringLiteral("manage-profiles")));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::KeyBindings))));
     m_menu->addAction(actionCollection()->action(QLatin1String(KStandardAction::name(KStandardAction::ConfigureNotifications))));
@@ -687,7 +707,7 @@ void MainWindow::updateWindowSizeMenus()
 
 void MainWindow::updateWindowWidthMenu()
 {
-    QAction* action = 0;
+    QAction* action = nullptr;
 
     if (m_windowWidthMenu->isEmpty())
     {
@@ -714,7 +734,7 @@ void MainWindow::updateWindowWidthMenu()
 
 void MainWindow::updateWindowHeightMenu()
 {
-    QAction* action = 0;
+    QAction* action = nullptr;
 
     if (m_windowHeightMenu->isEmpty())
     {
@@ -741,7 +761,38 @@ void MainWindow::updateWindowHeightMenu()
 
 void MainWindow::configureKeys()
 {
-    KShortcutsDialog::configure(actionCollection());
+    KShortcutsDialog dialog(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, this);
+    dialog.addCollection(actionCollection());
+
+    const auto collections = m_sessionStack->getPartActionCollections();
+
+    if (collections.size() >= 1) {
+        dialog.addCollection(collections.at(0), QStringLiteral("Konsolepart"));
+    }
+
+    if (!dialog.configure()) {
+        return;
+    }
+
+    if (collections.size() >= 1) {
+
+        // We need to update all the other collections
+        // rootCollection is the collection which got updatet by the dialog
+        const auto rootCollection = collections.at(0);
+
+        // For all the other collections
+        for (auto i = 1; i < collections.size(); ++i) {
+
+            // Update all the action they share with rootCollection
+            const auto rootActions = rootCollection->actions();
+            for (const auto* action : rootActions) {
+                if (auto* destaction = collections.at(i)->action(action->objectName())) {
+                    destaction->setShortcuts(action->shortcuts());
+                }
+            }
+
+        }
+    }
 }
 
 void MainWindow::configureNotifications()
@@ -805,6 +856,29 @@ void MainWindow::applySettings()
 
     m_tabBar->setVisible(Settings::showTabBar());
 
+    if (!Settings::showSystrayIcon() && m_notifierItem) {
+        delete m_notifierItem;
+        m_notifierItem = nullptr;
+
+        // Removing the notifier item deletes the menu
+        // add a new one
+        m_menu = new QMenu(this);
+        setupMenu();
+        m_titleBar->updateMenu();
+    } else if (Settings::showSystrayIcon() && !m_notifierItem) {
+        m_notifierItem = new KStatusNotifierItem(this);
+        m_notifierItem->setStandardActionsEnabled(false);
+        m_notifierItem->setIconByName(QStringLiteral("yakuake"));
+        m_notifierItem->setStatus(KStatusNotifierItem::Active);
+        m_notifierItem->setContextMenu(m_menu);
+
+        // Prevent the default implementation of showing
+        // and instead run toggleWindowState
+        m_notifierItem->setAssociatedWidget(nullptr);
+        connect(m_notifierItem, &KStatusNotifierItem::activateRequested, this, &MainWindow::toggleWindowState);
+        updateTrayTooltip();
+    }
+
     setKeepOpen(Settings::keepOpen());
 
     updateScreenMenu();
@@ -1042,7 +1116,7 @@ void MainWindow::wmActiveWindowChanged()
         return;
     }
 
-    KWindowInfo info(KWindowSystem::activeWindow(), 0, NET::WM2TransientFor);
+    KWindowInfo info(KWindowSystem::activeWindow(), {}, NET::WM2TransientFor);
 
     if (info.valid() && info.transientFor() == winId()) {
         return;
@@ -1096,7 +1170,32 @@ bool MainWindow::event(QEvent* event)
     return KMainWindow::event(event);
 }
 
+bool MainWindow::focusNextPrevChild(bool)
+{
+    return false;
+}
+
 void MainWindow::toggleWindowState()
+{
+    if (m_isWayland) {
+        auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.plasmashell"), QStringLiteral("/StrutManager"),
+                                                      QStringLiteral("org.kde.PlasmaShell.StrutManager"), QStringLiteral("availableScreenRect"));
+        message.setArguments({QGuiApplication::screens().at(getScreen())->name()});
+        QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(message);
+        QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
+
+        QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [=]() {
+            QDBusPendingReply<QRect> reply = *watcher;
+            m_availableScreenRect = reply.isValid() ? reply.value() : QRect();
+            _toggleWindowState();
+            watcher->deleteLater();
+        });
+    } else {
+        _toggleWindowState();
+    }
+}
+
+void MainWindow::_toggleWindowState()
 {
     bool visible = isVisible();
 
@@ -1182,7 +1281,7 @@ void MainWindow::kwinAssistToggleWindowState(bool visible)
     int count;
     Atom* list = XListProperties(display, DefaultRootWindow(display), &count);
 
-    if (list != NULL)
+    if (list != nullptr)
     {
         gotEffect = (std::find(list, list + count, atom) != list + count);
 
@@ -1363,7 +1462,7 @@ void MainWindow::sharedAfterHideWindow()
 
 #if HAVE_KWAYLAND
     delete m_plasmaShellSurface;
-    m_plasmaShellSurface = Q_NULLPTR;
+    m_plasmaShellSurface = nullptr;
 #endif
 
     emit windowClosed();
@@ -1425,15 +1524,28 @@ void MainWindow::setFullScreen(bool state)
 
 int MainWindow::getScreen()
 {
-    if (!Settings::screen())
-        return QApplication::desktop()->screenNumber(QCursor::pos());
-    else
+    if (!Settings::screen() || Settings::screen() > QGuiApplication::screens().length()) {
+        // Right after unplugging an external monitor and the Yakuake window was on
+        // that monitor, QGuiApplication::screenAt() can return nullptr so we fallback on
+        // the first monitor.
+        QScreen *screen = QGuiApplication::screenAt(QCursor::pos());
+        return screen ? QGuiApplication::screens().indexOf(screen) : 0;
+    } else {
         return Settings::screen() - 1;
+    }
+}
+
+QRect MainWindow::getScreenGeometry()
+{
+    QScreen *screen = QGuiApplication::screens().at(getScreen());
+    QRect screenGeometry = screen->geometry();
+    screenGeometry.moveTo(screenGeometry.topLeft() / screen->devicePixelRatio());
+    return screenGeometry;
 }
 
 QRect MainWindow::getDesktopGeometry()
 {
-    QRect screenGeometry = QApplication::desktop()->screenGeometry(getScreen());
+    QRect screenGeometry = getScreenGeometry();
 
     QAction* action = actionCollection()->action(QStringLiteral("view-full-screen"));
 
@@ -1441,8 +1553,9 @@ QRect MainWindow::getDesktopGeometry()
         return screenGeometry;
 
     if (m_isWayland) {
-        // on Wayland it's not possible to get the work area
-        return screenGeometry;
+        // on Wayland it's not possible to get the work area from KWindowSystem
+        // but plasmashell provides this through dbus
+        return m_availableScreenRect.isValid() ? m_availableScreenRect : screenGeometry;
     }
 
     if (QGuiApplication::screens().count() > 1)
@@ -1458,7 +1571,8 @@ QRect MainWindow::getDesktopGeometry()
 
             if (KWindowSystem::hasWId(windowId))
             {
-                KWindowInfo windowInfo = KWindowInfo(windowId, NET::WMDesktop, NET::WM2ExtendedStrut);
+                KWindowInfo windowInfo = KWindowInfo(windowId, NET::WMDesktop | NET::WMGeometry,
+                        NET::WM2ExtendedStrut);
 
                 // If windowInfo is valid and the window is located at the same (current)
                 // desktop with the yakuake window...
@@ -1470,19 +1584,23 @@ QRect MainWindow::getDesktopGeometry()
                     QRect topStrut(strut.top_start, 0, strut.top_end - strut.top_start, strut.top_width);
                     QRect bottomStrut(strut.bottom_start, screenGeometry.bottom() - strut.bottom_width,
                                       strut.bottom_end - strut.bottom_start, strut.bottom_width);
-                    QRect leftStrut(0, strut.left_width, strut.left_start, strut.left_end - strut.left_start);
+                    QRect leftStrut(0, strut.left_start, strut.left_width, strut.left_end - strut.left_start);
                     QRect rightStrut(screenGeometry.right() - strut.right_width, strut.right_start,
-                                     strut.right_end - strut.right_start, strut.right_width);
+                                     strut.right_width, strut.right_end - strut.right_start);
 
                     // If the window has no strut, no need to bother further.
                     if (topStrut.isEmpty() && bottomStrut.isEmpty() && leftStrut.isEmpty() && rightStrut.isEmpty())
                         continue;
 
-                    // If any of the strut intersects with our screen geometry, it will be correctly handled
-                    // by workArea().
-                    if (topStrut.intersects(screenGeometry) || bottomStrut.intersects(screenGeometry) ||
+                    // If any of the strut and the window itself intersects with our screen geometry,
+                    // it will be correctly handled by workArea(). If the window doesn't intersect
+                    // with our screen geometry it's most likely a plasma panel and can/should be
+                    // ignored
+                    if ((topStrut.intersects(screenGeometry) || bottomStrut.intersects(screenGeometry) ||
                         leftStrut.intersects(screenGeometry) || rightStrut.intersects(screenGeometry))
-                        continue;
+                            && windowInfo.geometry().intersects(screenGeometry)) {
+                            continue;
+                    }
 
                     // This window has a strut on the same desktop as us but which does not cover our screen
                     // geometry. It should be ignored, otherwise the returned work area will wrongly include
@@ -1503,21 +1621,6 @@ void MainWindow::whatsThis()
     QWhatsThis::enterWhatsThisMode();
 }
 
-void MainWindow::showStartupPopup()
-{
-    QAction* action = static_cast<QAction*>(actionCollection()->action(QStringLiteral("toggle-window-state")));
-
-    const QList<QKeySequence> &shortcuts = KGlobalAccel::self()->shortcut(action);
-
-    if (shortcuts.isEmpty())
-        return;
-
-    QString shortcut(shortcuts.first().toString(QKeySequence::NativeText));
-    QString message(xi18nc("@info", "Application successfully started.<nl/>" "Press <shortcut>%1</shortcut> to use it ...", shortcut));
-
-    KNotification::event(QLatin1String("startup"), message, QPixmap(), this);
-}
-
 void MainWindow::showFirstRunDialog()
 {
     if (!m_firstRunDialog)
@@ -1555,3 +1658,19 @@ void MainWindow::updateUseTranslucency()
 {
     m_useTranslucency = (Settings::translucency() && KWindowSystem::compositingActive());
 }
+
+void MainWindow::updateTrayTooltip()
+{
+    if (!m_notifierItem) {
+        return;
+    }
+
+    auto* action = actionCollection()->action(QStringLiteral("toggle-window-state"));
+    const QList<QKeySequence> &shortcuts = KGlobalAccel::self()->shortcut(action);
+    if (!shortcuts.isEmpty()) {
+        const QString shortcut(shortcuts.first().toString(QKeySequence::NativeText));
+        m_notifierItem->setToolTip(QStringLiteral("yakuake"),
+                QStringLiteral("Yakuake"), xi18nc("@info", "Press <shortcut>%1</shortcut> to open", shortcut));
+    }
+
+}
